// Code generated by ent, DO NOT EDIT.

package ent

import (
	"context"
	"errors"
	"fmt"
	"log"

	"git.dotya.ml/mirre-mt/pcmt/ent/migrate"
	"github.com/google/uuid"

	"entgo.io/ent"
	"entgo.io/ent/dialect"
	"entgo.io/ent/dialect/sql"
	"git.dotya.ml/mirre-mt/pcmt/ent/hibp"
	"git.dotya.ml/mirre-mt/pcmt/ent/setup"
	"git.dotya.ml/mirre-mt/pcmt/ent/user"
)

// Client is the client that holds all ent builders.
type Client struct {
	config
	// Schema is the client for creating, migrating and dropping schema.
	Schema *migrate.Schema
	// HIBP is the client for interacting with the HIBP builders.
	HIBP *HIBPClient
	// Setup is the client for interacting with the Setup builders.
	Setup *SetupClient
	// User is the client for interacting with the User builders.
	User *UserClient
}

// NewClient creates a new client configured with the given options.
func NewClient(opts ...Option) *Client {
	cfg := config{log: log.Println, hooks: &hooks{}, inters: &inters{}}
	cfg.options(opts...)
	client := &Client{config: cfg}
	client.init()
	return client
}

func (c *Client) init() {
	c.Schema = migrate.NewSchema(c.driver)
	c.HIBP = NewHIBPClient(c.config)
	c.Setup = NewSetupClient(c.config)
	c.User = NewUserClient(c.config)
}

type (
	// config is the configuration for the client and its builder.
	config struct {
		// driver used for executing database requests.
		driver dialect.Driver
		// debug enable a debug logging.
		debug bool
		// log used for logging on debug mode.
		log func(...any)
		// hooks to execute on mutations.
		hooks *hooks
		// interceptors to execute on queries.
		inters *inters
	}
	// Option function to configure the client.
	Option func(*config)
)

// options applies the options on the config object.
func (c *config) options(opts ...Option) {
	for _, opt := range opts {
		opt(c)
	}
	if c.debug {
		c.driver = dialect.Debug(c.driver, c.log)
	}
}

// Debug enables debug logging on the ent.Driver.
func Debug() Option {
	return func(c *config) {
		c.debug = true
	}
}

// Log sets the logging function for debug mode.
func Log(fn func(...any)) Option {
	return func(c *config) {
		c.log = fn
	}
}

// Driver configures the client driver.
func Driver(driver dialect.Driver) Option {
	return func(c *config) {
		c.driver = driver
	}
}

// Open opens a database/sql.DB specified by the driver name and
// the data source name, and returns a new client attached to it.
// Optional parameters can be added for configuring the client.
func Open(driverName, dataSourceName string, options ...Option) (*Client, error) {
	switch driverName {
	case dialect.MySQL, dialect.Postgres, dialect.SQLite:
		drv, err := sql.Open(driverName, dataSourceName)
		if err != nil {
			return nil, err
		}
		return NewClient(append(options, Driver(drv))...), nil
	default:
		return nil, fmt.Errorf("unsupported driver: %q", driverName)
	}
}

// Tx returns a new transactional client. The provided context
// is used until the transaction is committed or rolled back.
func (c *Client) Tx(ctx context.Context) (*Tx, error) {
	if _, ok := c.driver.(*txDriver); ok {
		return nil, errors.New("ent: cannot start a transaction within a transaction")
	}
	tx, err := newTx(ctx, c.driver)
	if err != nil {
		return nil, fmt.Errorf("ent: starting a transaction: %w", err)
	}
	cfg := c.config
	cfg.driver = tx
	return &Tx{
		ctx:    ctx,
		config: cfg,
		HIBP:   NewHIBPClient(cfg),
		Setup:  NewSetupClient(cfg),
		User:   NewUserClient(cfg),
	}, nil
}

// BeginTx returns a transactional client with specified options.
func (c *Client) BeginTx(ctx context.Context, opts *sql.TxOptions) (*Tx, error) {
	if _, ok := c.driver.(*txDriver); ok {
		return nil, errors.New("ent: cannot start a transaction within a transaction")
	}
	tx, err := c.driver.(interface {
		BeginTx(context.Context, *sql.TxOptions) (dialect.Tx, error)
	}).BeginTx(ctx, opts)
	if err != nil {
		return nil, fmt.Errorf("ent: starting a transaction: %w", err)
	}
	cfg := c.config
	cfg.driver = &txDriver{tx: tx, drv: c.driver}
	return &Tx{
		ctx:    ctx,
		config: cfg,
		HIBP:   NewHIBPClient(cfg),
		Setup:  NewSetupClient(cfg),
		User:   NewUserClient(cfg),
	}, nil
}

// Debug returns a new debug-client. It's used to get verbose logging on specific operations.
//
//	client.Debug().
//		HIBP.
//		Query().
//		Count(ctx)
func (c *Client) Debug() *Client {
	if c.debug {
		return c
	}
	cfg := c.config
	cfg.driver = dialect.Debug(c.driver, c.log)
	client := &Client{config: cfg}
	client.init()
	return client
}

// Close closes the database connection and prevents new queries from starting.
func (c *Client) Close() error {
	return c.driver.Close()
}

// Use adds the mutation hooks to all the entity clients.
// In order to add hooks to a specific client, call: `client.Node.Use(...)`.
func (c *Client) Use(hooks ...Hook) {
	c.HIBP.Use(hooks...)
	c.Setup.Use(hooks...)
	c.User.Use(hooks...)
}

// Intercept adds the query interceptors to all the entity clients.
// In order to add interceptors to a specific client, call: `client.Node.Intercept(...)`.
func (c *Client) Intercept(interceptors ...Interceptor) {
	c.HIBP.Intercept(interceptors...)
	c.Setup.Intercept(interceptors...)
	c.User.Intercept(interceptors...)
}

// Mutate implements the ent.Mutator interface.
func (c *Client) Mutate(ctx context.Context, m Mutation) (Value, error) {
	switch m := m.(type) {
	case *HIBPMutation:
		return c.HIBP.mutate(ctx, m)
	case *SetupMutation:
		return c.Setup.mutate(ctx, m)
	case *UserMutation:
		return c.User.mutate(ctx, m)
	default:
		return nil, fmt.Errorf("ent: unknown mutation type %T", m)
	}
}

// HIBPClient is a client for the HIBP schema.
type HIBPClient struct {
	config
}

// NewHIBPClient returns a client for the HIBP from the given config.
func NewHIBPClient(c config) *HIBPClient {
	return &HIBPClient{config: c}
}

// Use adds a list of mutation hooks to the hooks stack.
// A call to `Use(f, g, h)` equals to `hibp.Hooks(f(g(h())))`.
func (c *HIBPClient) Use(hooks ...Hook) {
	c.hooks.HIBP = append(c.hooks.HIBP, hooks...)
}

// Intercept adds a list of query interceptors to the interceptors stack.
// A call to `Intercept(f, g, h)` equals to `hibp.Intercept(f(g(h())))`.
func (c *HIBPClient) Intercept(interceptors ...Interceptor) {
	c.inters.HIBP = append(c.inters.HIBP, interceptors...)
}

// Create returns a builder for creating a HIBP entity.
func (c *HIBPClient) Create() *HIBPCreate {
	mutation := newHIBPMutation(c.config, OpCreate)
	return &HIBPCreate{config: c.config, hooks: c.Hooks(), mutation: mutation}
}

// CreateBulk returns a builder for creating a bulk of HIBP entities.
func (c *HIBPClient) CreateBulk(builders ...*HIBPCreate) *HIBPCreateBulk {
	return &HIBPCreateBulk{config: c.config, builders: builders}
}

// Update returns an update builder for HIBP.
func (c *HIBPClient) Update() *HIBPUpdate {
	mutation := newHIBPMutation(c.config, OpUpdate)
	return &HIBPUpdate{config: c.config, hooks: c.Hooks(), mutation: mutation}
}

// UpdateOne returns an update builder for the given entity.
func (c *HIBPClient) UpdateOne(h *HIBP) *HIBPUpdateOne {
	mutation := newHIBPMutation(c.config, OpUpdateOne, withHIBP(h))
	return &HIBPUpdateOne{config: c.config, hooks: c.Hooks(), mutation: mutation}
}

// UpdateOneID returns an update builder for the given id.
func (c *HIBPClient) UpdateOneID(id uuid.UUID) *HIBPUpdateOne {
	mutation := newHIBPMutation(c.config, OpUpdateOne, withHIBPID(id))
	return &HIBPUpdateOne{config: c.config, hooks: c.Hooks(), mutation: mutation}
}

// Delete returns a delete builder for HIBP.
func (c *HIBPClient) Delete() *HIBPDelete {
	mutation := newHIBPMutation(c.config, OpDelete)
	return &HIBPDelete{config: c.config, hooks: c.Hooks(), mutation: mutation}
}

// DeleteOne returns a builder for deleting the given entity.
func (c *HIBPClient) DeleteOne(h *HIBP) *HIBPDeleteOne {
	return c.DeleteOneID(h.ID)
}

// DeleteOneID returns a builder for deleting the given entity by its id.
func (c *HIBPClient) DeleteOneID(id uuid.UUID) *HIBPDeleteOne {
	builder := c.Delete().Where(hibp.ID(id))
	builder.mutation.id = &id
	builder.mutation.op = OpDeleteOne
	return &HIBPDeleteOne{builder}
}

// Query returns a query builder for HIBP.
func (c *HIBPClient) Query() *HIBPQuery {
	return &HIBPQuery{
		config: c.config,
		ctx:    &QueryContext{Type: TypeHIBP},
		inters: c.Interceptors(),
	}
}

// Get returns a HIBP entity by its id.
func (c *HIBPClient) Get(ctx context.Context, id uuid.UUID) (*HIBP, error) {
	return c.Query().Where(hibp.ID(id)).Only(ctx)
}

// GetX is like Get, but panics if an error occurs.
func (c *HIBPClient) GetX(ctx context.Context, id uuid.UUID) *HIBP {
	obj, err := c.Get(ctx, id)
	if err != nil {
		panic(err)
	}
	return obj
}

// Hooks returns the client hooks.
func (c *HIBPClient) Hooks() []Hook {
	return c.hooks.HIBP
}

// Interceptors returns the client interceptors.
func (c *HIBPClient) Interceptors() []Interceptor {
	return c.inters.HIBP
}

func (c *HIBPClient) mutate(ctx context.Context, m *HIBPMutation) (Value, error) {
	switch m.Op() {
	case OpCreate:
		return (&HIBPCreate{config: c.config, hooks: c.Hooks(), mutation: m}).Save(ctx)
	case OpUpdate:
		return (&HIBPUpdate{config: c.config, hooks: c.Hooks(), mutation: m}).Save(ctx)
	case OpUpdateOne:
		return (&HIBPUpdateOne{config: c.config, hooks: c.Hooks(), mutation: m}).Save(ctx)
	case OpDelete, OpDeleteOne:
		return (&HIBPDelete{config: c.config, hooks: c.Hooks(), mutation: m}).Exec(ctx)
	default:
		return nil, fmt.Errorf("ent: unknown HIBP mutation op: %q", m.Op())
	}
}

// SetupClient is a client for the Setup schema.
type SetupClient struct {
	config
}

// NewSetupClient returns a client for the Setup from the given config.
func NewSetupClient(c config) *SetupClient {
	return &SetupClient{config: c}
}

// Use adds a list of mutation hooks to the hooks stack.
// A call to `Use(f, g, h)` equals to `setup.Hooks(f(g(h())))`.
func (c *SetupClient) Use(hooks ...Hook) {
	c.hooks.Setup = append(c.hooks.Setup, hooks...)
}

// Intercept adds a list of query interceptors to the interceptors stack.
// A call to `Intercept(f, g, h)` equals to `setup.Intercept(f(g(h())))`.
func (c *SetupClient) Intercept(interceptors ...Interceptor) {
	c.inters.Setup = append(c.inters.Setup, interceptors...)
}

// Create returns a builder for creating a Setup entity.
func (c *SetupClient) Create() *SetupCreate {
	mutation := newSetupMutation(c.config, OpCreate)
	return &SetupCreate{config: c.config, hooks: c.Hooks(), mutation: mutation}
}

// CreateBulk returns a builder for creating a bulk of Setup entities.
func (c *SetupClient) CreateBulk(builders ...*SetupCreate) *SetupCreateBulk {
	return &SetupCreateBulk{config: c.config, builders: builders}
}

// Update returns an update builder for Setup.
func (c *SetupClient) Update() *SetupUpdate {
	mutation := newSetupMutation(c.config, OpUpdate)
	return &SetupUpdate{config: c.config, hooks: c.Hooks(), mutation: mutation}
}

// UpdateOne returns an update builder for the given entity.
func (c *SetupClient) UpdateOne(s *Setup) *SetupUpdateOne {
	mutation := newSetupMutation(c.config, OpUpdateOne, withSetup(s))
	return &SetupUpdateOne{config: c.config, hooks: c.Hooks(), mutation: mutation}
}

// UpdateOneID returns an update builder for the given id.
func (c *SetupClient) UpdateOneID(id uuid.UUID) *SetupUpdateOne {
	mutation := newSetupMutation(c.config, OpUpdateOne, withSetupID(id))
	return &SetupUpdateOne{config: c.config, hooks: c.Hooks(), mutation: mutation}
}

// Delete returns a delete builder for Setup.
func (c *SetupClient) Delete() *SetupDelete {
	mutation := newSetupMutation(c.config, OpDelete)
	return &SetupDelete{config: c.config, hooks: c.Hooks(), mutation: mutation}
}

// DeleteOne returns a builder for deleting the given entity.
func (c *SetupClient) DeleteOne(s *Setup) *SetupDeleteOne {
	return c.DeleteOneID(s.ID)
}

// DeleteOneID returns a builder for deleting the given entity by its id.
func (c *SetupClient) DeleteOneID(id uuid.UUID) *SetupDeleteOne {
	builder := c.Delete().Where(setup.ID(id))
	builder.mutation.id = &id
	builder.mutation.op = OpDeleteOne
	return &SetupDeleteOne{builder}
}

// Query returns a query builder for Setup.
func (c *SetupClient) Query() *SetupQuery {
	return &SetupQuery{
		config: c.config,
		ctx:    &QueryContext{Type: TypeSetup},
		inters: c.Interceptors(),
	}
}

// Get returns a Setup entity by its id.
func (c *SetupClient) Get(ctx context.Context, id uuid.UUID) (*Setup, error) {
	return c.Query().Where(setup.ID(id)).Only(ctx)
}

// GetX is like Get, but panics if an error occurs.
func (c *SetupClient) GetX(ctx context.Context, id uuid.UUID) *Setup {
	obj, err := c.Get(ctx, id)
	if err != nil {
		panic(err)
	}
	return obj
}

// Hooks returns the client hooks.
func (c *SetupClient) Hooks() []Hook {
	return c.hooks.Setup
}

// Interceptors returns the client interceptors.
func (c *SetupClient) Interceptors() []Interceptor {
	return c.inters.Setup
}

func (c *SetupClient) mutate(ctx context.Context, m *SetupMutation) (Value, error) {
	switch m.Op() {
	case OpCreate:
		return (&SetupCreate{config: c.config, hooks: c.Hooks(), mutation: m}).Save(ctx)
	case OpUpdate:
		return (&SetupUpdate{config: c.config, hooks: c.Hooks(), mutation: m}).Save(ctx)
	case OpUpdateOne:
		return (&SetupUpdateOne{config: c.config, hooks: c.Hooks(), mutation: m}).Save(ctx)
	case OpDelete, OpDeleteOne:
		return (&SetupDelete{config: c.config, hooks: c.Hooks(), mutation: m}).Exec(ctx)
	default:
		return nil, fmt.Errorf("ent: unknown Setup mutation op: %q", m.Op())
	}
}

// UserClient is a client for the User schema.
type UserClient struct {
	config
}

// NewUserClient returns a client for the User from the given config.
func NewUserClient(c config) *UserClient {
	return &UserClient{config: c}
}

// Use adds a list of mutation hooks to the hooks stack.
// A call to `Use(f, g, h)` equals to `user.Hooks(f(g(h())))`.
func (c *UserClient) Use(hooks ...Hook) {
	c.hooks.User = append(c.hooks.User, hooks...)
}

// Intercept adds a list of query interceptors to the interceptors stack.
// A call to `Intercept(f, g, h)` equals to `user.Intercept(f(g(h())))`.
func (c *UserClient) Intercept(interceptors ...Interceptor) {
	c.inters.User = append(c.inters.User, interceptors...)
}

// Create returns a builder for creating a User entity.
func (c *UserClient) Create() *UserCreate {
	mutation := newUserMutation(c.config, OpCreate)
	return &UserCreate{config: c.config, hooks: c.Hooks(), mutation: mutation}
}

// CreateBulk returns a builder for creating a bulk of User entities.
func (c *UserClient) CreateBulk(builders ...*UserCreate) *UserCreateBulk {
	return &UserCreateBulk{config: c.config, builders: builders}
}

// Update returns an update builder for User.
func (c *UserClient) Update() *UserUpdate {
	mutation := newUserMutation(c.config, OpUpdate)
	return &UserUpdate{config: c.config, hooks: c.Hooks(), mutation: mutation}
}

// UpdateOne returns an update builder for the given entity.
func (c *UserClient) UpdateOne(u *User) *UserUpdateOne {
	mutation := newUserMutation(c.config, OpUpdateOne, withUser(u))
	return &UserUpdateOne{config: c.config, hooks: c.Hooks(), mutation: mutation}
}

// UpdateOneID returns an update builder for the given id.
func (c *UserClient) UpdateOneID(id uuid.UUID) *UserUpdateOne {
	mutation := newUserMutation(c.config, OpUpdateOne, withUserID(id))
	return &UserUpdateOne{config: c.config, hooks: c.Hooks(), mutation: mutation}
}

// Delete returns a delete builder for User.
func (c *UserClient) Delete() *UserDelete {
	mutation := newUserMutation(c.config, OpDelete)
	return &UserDelete{config: c.config, hooks: c.Hooks(), mutation: mutation}
}

// DeleteOne returns a builder for deleting the given entity.
func (c *UserClient) DeleteOne(u *User) *UserDeleteOne {
	return c.DeleteOneID(u.ID)
}

// DeleteOneID returns a builder for deleting the given entity by its id.
func (c *UserClient) DeleteOneID(id uuid.UUID) *UserDeleteOne {
	builder := c.Delete().Where(user.ID(id))
	builder.mutation.id = &id
	builder.mutation.op = OpDeleteOne
	return &UserDeleteOne{builder}
}

// Query returns a query builder for User.
func (c *UserClient) Query() *UserQuery {
	return &UserQuery{
		config: c.config,
		ctx:    &QueryContext{Type: TypeUser},
		inters: c.Interceptors(),
	}
}

// Get returns a User entity by its id.
func (c *UserClient) Get(ctx context.Context, id uuid.UUID) (*User, error) {
	return c.Query().Where(user.ID(id)).Only(ctx)
}

// GetX is like Get, but panics if an error occurs.
func (c *UserClient) GetX(ctx context.Context, id uuid.UUID) *User {
	obj, err := c.Get(ctx, id)
	if err != nil {
		panic(err)
	}
	return obj
}

// Hooks returns the client hooks.
func (c *UserClient) Hooks() []Hook {
	return c.hooks.User
}

// Interceptors returns the client interceptors.
func (c *UserClient) Interceptors() []Interceptor {
	return c.inters.User
}

func (c *UserClient) mutate(ctx context.Context, m *UserMutation) (Value, error) {
	switch m.Op() {
	case OpCreate:
		return (&UserCreate{config: c.config, hooks: c.Hooks(), mutation: m}).Save(ctx)
	case OpUpdate:
		return (&UserUpdate{config: c.config, hooks: c.Hooks(), mutation: m}).Save(ctx)
	case OpUpdateOne:
		return (&UserUpdateOne{config: c.config, hooks: c.Hooks(), mutation: m}).Save(ctx)
	case OpDelete, OpDeleteOne:
		return (&UserDelete{config: c.config, hooks: c.Hooks(), mutation: m}).Exec(ctx)
	default:
		return nil, fmt.Errorf("ent: unknown User mutation op: %q", m.Op())
	}
}

// hooks and interceptors per client, for fast access.
type (
	hooks struct {
		HIBP, Setup, User []ent.Hook
	}
	inters struct {
		HIBP, Setup, User []ent.Interceptor
	}
)
